pub mod session {
    use std::error::Error;
    use rand::{distributions::Alphanumeric, Rng};
    use serde::{Deserialize, Serialize};

    use crate::discord::discord::User;

    #[derive(Clone, Debug, Deserialize, Serialize)]
    pub struct Session {
        pub access_token: String,
        pub user: User,
        pub session_token: String,
    }

    pub struct SessionManager {
        pub sessions: Vec<Session>,
    }

    impl SessionManager {
        pub async fn generate_random_string(&mut self) -> String {
            rand::thread_rng()
                .sample_iter(&Alphanumeric)
                .take(32)
                .map(char::from)
                .collect()
        }

        pub fn new() -> Self {
            Self {
                sessions: Vec::new(),
            }
        }

        pub async fn get_session(&mut self, session: &String) -> Result<Session, Box<dyn Error>> {
            let found_sessions: Vec<Session> = self
                .sessions
                .clone()
                .into_iter()
                .filter(|s| s.session_token.eq(session))
                .collect();
            Ok(found_sessions.get(0).unwrap().clone())
        }

        pub async fn generate_session(&mut self, user: User, access_token: &str) -> Session {
            self.sessions.retain(|s| s.user.id != user.id);
            let token = self.generate_random_string().await;
            let session = Session {
                access_token: access_token.to_string(),
                user,
                session_token: token,
            };
            self.sessions.push(session.clone());
            session
        }
    }
}
